<?php
class MyClass {}
class YourClass {}
interface MyInterface {}
interface YourInterface {}
trait MyTrait {}
trait YourTrait {}
enum MyEnum {}
enum YourEnum {}
class MyClass {}
interface MyInterface {}
trait MyTrait {}
enum MyEnum {}
?>
